@@ -115,6 +115,10 @@ fmapConst :: f b -> (a -> b) -> f a -> f b
 fmapConst x _ _ = x
 {-# INLINE fmapConst #-}
 
+replaceConst :: f a -> a -> f b -> f a
+replaceConst x _ _ = x
+{-# INLINE replaceConst #-}
+
 foldrConst :: b -> (a -> b -> b) -> b -> t a -> b
 foldrConst x _ _ _ = x
 {-# INLINE foldrConst #-}
@@ -123,6 +127,10 @@ foldMapConst :: m -> (a -> m) -> t a -> m
 foldMapConst x _ _ = x
 {-# INLINE foldMapConst #-}
 
+nullConst :: Bool -> t a -> Bool
+nullConst x _ = x
+{-# INLINE nullConst #-}
+
 traverseConst :: f (t b) -> (a -> f b) -> t a -> f (t b)
 traverseConst x _ _ = x
 {-# INLINE traverseConst #-}
@@ -670,21 +678,6 @@ isTrue# x = x
 #endif
 {-# INLINE isTrue# #-}
 
--- isRight and fromEither taken from the extra package (BSD3-licensed)
-
--- | Test if an 'Either' value is the 'Right' constructor.
---   Provided as standard with GHC 7.8 and above.
-isRight :: Either l r -> Bool
-isRight Right{} = True; isRight _ = False
-
--- | Pull the value out of an 'Either' where both alternatives
---   have the same type.
---
--- > \x -> fromEither (Left x ) == x
--- > \x -> fromEither (Right x) == x
-fromEither :: Either a a -> a
-fromEither = either id id
-
 -- filterByList, filterByLists, and partitionByList taken from GHC (BSD3-licensed)
 
 -- | 'filterByList' takes a list of Bools and a list of some elements and
@@ -726,15 +719,6 @@ partitionByList = go [] []
     go trues falses (False : bs) (x : xs) = go trues (x:falses) bs xs
     go trues falses _ _ = (reverse trues, reverse falses)
 
--- | Apply an @Either Exp Exp@ expression to an 'Exp' expression,
--- preserving the 'Either'-ness.
-appEitherE :: Q (Either Exp Exp) -> Q Exp -> Q (Either Exp Exp)
-appEitherE e1Q e2Q = do
-    e2 <- e2Q
-    let e2' :: Exp -> Exp
-        e2' = (`AppE` e2)
-    either (Left . e2') (Right . e2') `fmap` e1Q
-
 integerE :: Int -> Q Exp
 integerE = litE . integerL . fromIntegral
 
@@ -903,23 +887,52 @@ isTyVar (VarT _)   = True
 isTyVar (SigT t _) = isTyVar t
 isTyVar _          = False
 
--- | Is the given type a type family constructor (and not a data family constructor)?
-isTyFamily :: Type -> Q Bool
-isTyFamily (ConT n) = do
-    info <- reify n
-    return $ case info of
+-- | Detect if a Name in a list of provided Names occurs as an argument to some
+-- type family. This makes an effort to exclude /oversaturated/ arguments to
+-- type families. For instance, if one declared the following type family:
+--
+-- @
+-- type family F a :: Type -> Type
+-- @
+--
+-- Then in the type @F a b@, we would consider @a@ to be an argument to @F@,
+-- but not @b@.
+isInTypeFamilyApp :: [Name] -> Type -> [Type] -> Q Bool
+isInTypeFamilyApp names tyFun tyArgs =
+  case tyFun of
+    ConT tcName -> go tcName
+    _           -> return False
+  where
+    go :: Name -> Q Bool
+    go tcName = do
+      info <- reify tcName
+      case info of
 #if MIN_VERSION_template_haskell(2,11,0)
-         FamilyI OpenTypeFamilyD{} _       -> True
+        FamilyI (OpenTypeFamilyD (TypeFamilyHead _ bndrs _ _)) _
+          -> withinFirstArgs bndrs
 #elif MIN_VERSION_template_haskell(2,7,0)
-         FamilyI (FamilyD TypeFam _ _ _) _ -> True
+        FamilyI (FamilyD TypeFam _ bndrs _) _
+          -> withinFirstArgs bndrs
 #else
-         TyConI  (FamilyD TypeFam _ _ _)   -> True
+        TyConI (FamilyD TypeFam _ bndrs _)
+          -> withinFirstArgs bndrs
 #endif
-#if MIN_VERSION_template_haskell(2,9,0)
-         FamilyI ClosedTypeFamilyD{} _     -> True
+
+#if MIN_VERSION_template_haskell(2,11,0)
+        FamilyI (ClosedTypeFamilyD (TypeFamilyHead _ bndrs _ _) _) _
+          -> withinFirstArgs bndrs
+#elif MIN_VERSION_template_haskell(2,9,0)
+        FamilyI (ClosedTypeFamilyD _ bndrs _ _) _
+          -> withinFirstArgs bndrs
 #endif
-         _ -> False
-isTyFamily _ = return False
+
+        _ -> return False
+      where
+        withinFirstArgs :: [a] -> Q Bool
+        withinFirstArgs bndrs =
+          let firstArgs = take (length bndrs) tyArgs
+              argFVs    = freeVariables firstArgs
+          in return $ any (`elem` argFVs) names
 
 -- | Are all of the items in a list (which have an ordering) distinct?
 --
@@ -974,14 +987,17 @@ applyTyCon = applyTy . ConT
 -- @
 -- [Either, Int, Char]
 -- @
-unapplyTy :: Type -> [Type]
-unapplyTy = reverse . go
+unapplyTy :: Type -> (Type, [Type])
+unapplyTy ty = go ty ty []
   where
-    go :: Type -> [Type]
-    go (AppT t1 t2)    = t2:go t1
-    go (SigT t _)      = go t
-    go (ForallT _ _ t) = go t
-    go t               = [t]
+    go :: Type -> Type -> [Type] -> (Type, [Type])
+    go _      (AppT ty1 ty2)     args = go ty1 ty1 (ty2:args)
+    go origTy (SigT ty' _)       args = go origTy ty' args
+#if MIN_VERSION_template_haskell(2,11,0)
+    go origTy (InfixT ty1 n ty2) args = go origTy (ConT n `AppT` ty1 `AppT` ty2) args
+    go origTy (ParensT ty')      args = go origTy ty' args
+#endif
+    go origTy _                  args = (origTy, args)
 
 -- | Split a type signature by the arrows on its spine. For example, this:
 --
@@ -1161,16 +1177,6 @@ freshenType t =
      subst <- T.sequence (Map.fromList xs)
      return (applySubstitution subst t)
 
--- | Gets all of the required type variable binders mentioned in a Type.
-requiredTyVarsOfType :: Type -> [TyVarBndr]
-requiredTyVarsOfType = go
-  where
-    go :: Type -> [TyVarBndr]
-    go (AppT t1 t2) = go t1 ++ go t2
-    go (SigT t _)   = go t
-    go (VarT n)     = [PlainTV n]
-    go _            = []
-
 enumFromToExpr :: Q Exp -> Q Exp -> Q Exp
 enumFromToExpr f t = varE enumFromToValName `appE` f `appE` t
 
@@ -1247,12 +1253,18 @@ isTrueHashValName = mkDerivingCompatName_v "isTrue#"
 fmapConstValName :: Name
 fmapConstValName = mkDerivingCompatName_v "fmapConst"
 
+replaceConstValName :: Name
+replaceConstValName = mkDerivingCompatName_v "replaceConst"
+
 foldrConstValName :: Name
 foldrConstValName = mkDerivingCompatName_v "foldrConst"
 
 foldMapConstValName :: Name
 foldMapConstValName = mkDerivingCompatName_v "foldMapConst"
 
+nullConstValName :: Name
+nullConstValName = mkDerivingCompatName_v "nullConst"
+
 traverseConstValName :: Name
 traverseConstValName = mkDerivingCompatName_v "traverseConst"
 
@@ -1627,6 +1639,9 @@ readS_to_PrecValName = mkNameG_v "base" "Text.ParserCombinators.ReadPrec" "readS
 readsPrecValName :: Name
 readsPrecValName = mkNameG_v "base" "GHC.Read" "readsPrec"
 
+replaceValName :: Name
+replaceValName = mkNameG_v "base" "GHC.Base" "<$"
+
 resetValName :: Name
 resetValName = mkNameG_v "base" "Text.ParserCombinators.ReadPrec" "reset"
 
@@ -1792,11 +1807,6 @@ wHashDataName :: Name
 wHashDataName = mkNameG_d "base" "GHC.Word" "W#"
 #endif
 
-#if MIN_VERSION_base(4,6,0) && !(MIN_VERSION_base(4,9,0))
-starKindName :: Name
-starKindName = mkNameG_tc "ghc-prim" "GHC.Prim" "*"
-#endif
-
 #if MIN_VERSION_base(4,7,0)
 expectPValName :: Name
 expectPValName = mkNameG_v "base" "GHC.Read" "expectP"
@@ -1811,12 +1821,15 @@ expectPValName = mkDerivingCompatName_v "expectP"
 #endif
 
 #if MIN_VERSION_base(4,8,0)
-pureValName :: Name
-pureValName = mkNameG_v "base" "GHC.Base" "pure"
+allValName :: Name
+allValName = mkNameG_v "base" "Data.Foldable" "all"
 
 apValName :: Name
 apValName = mkNameG_v "base" "GHC.Base" "<*>"
 
+pureValName :: Name
+pureValName = mkNameG_v "base" "GHC.Base" "pure"
+
 liftA2ValName :: Name
 liftA2ValName = mkNameG_v "base" "GHC.Base" "liftA2"
 
@@ -1825,13 +1838,19 @@ mappendValName = mkNameG_v "base" "GHC.Base" "mappend"
 
 memptyValName :: Name
 memptyValName = mkNameG_v "base" "GHC.Base" "mempty"
+
+nullValName :: Name
+nullValName = mkNameG_v "base" "Data.Foldable" "null"
 #else
-pureValName :: Name
-pureValName = mkNameG_v "base" "Control.Applicative" "pure"
+allValName :: Name
+allValName = mkNameG_v "base" "GHC.List" "all"
 
 apValName :: Name
 apValName = mkNameG_v "base" "Control.Applicative" "<*>"
 
+pureValName :: Name
+pureValName = mkNameG_v "base" "Control.Applicative" "pure"
+
 liftA2ValName :: Name
 liftA2ValName = mkNameG_v "base" "Control.Applicative" "liftA2"
 
@@ -1840,6 +1859,9 @@ mappendValName = mkNameG_v "base" "Data.Monoid" "mappend"
 
 memptyValName :: Name
 memptyValName = mkNameG_v "base" "Data.Monoid" "mempty"
+
+nullValName :: Name
+nullValName = mkNameG_v "base" "GHC.List" "null"
 #endif
 
 #if MIN_VERSION_base(4,9,0)
@@ -2015,7 +2037,7 @@ makeFmapApply pos cRep conName (SigT ty _) name = makeFmapApply pos cRep conName
 makeFmapApply pos cRep conName t name = do
     let tyCon :: Type
         tyArgs :: [Type]
-        tyCon:tyArgs = unapplyTy t
+        (tyCon, tyArgs) = unapplyTy t
 
         numLastArgs :: Int
         numLastArgs = min (arity cRep) (length tyArgs)
@@ -2033,9 +2055,8 @@ makeFmapApply pos cRep conName t name = do
                                     (if pos then varE unApplyValName
                                             else makeFmapApply pos cRep conName beta name)
 
-    itf <- isTyFamily tyCon
-    if any (`mentionsName` [name]) lhsArgs
-          || itf && any (`mentionsName` [name]) tyArgs
+    itf <- isInTypeFamilyApp [name] tyCon tyArgs
+    if any (`mentionsName` [name]) lhsArgs || itf
        then outOfPlaceTyVarError cRep conName
        else inspectTy (head rhsArgs)
 
