@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-
 {-|
 Module:      Data.Enum.Deriving.Internal
 Copyright:   (C) 2015-2017 Ryan Scott
@@ -41,11 +39,7 @@ deriveEnum name = do
   case info of
     DatatypeInfo { datatypeContext   = ctxt
                  , datatypeName      = parentName
-#if MIN_VERSION_th_abstraction(0,3,0)
                  , datatypeInstTypes = instTypes
-#else
-                 , datatypeVars      = instTypes
-#endif
                  , datatypeVariant   = variant
                  , datatypeCons      = cons
                  } -> do
@@ -111,11 +105,7 @@ makeEnumFun ef name = do
   case info of
     DatatypeInfo { datatypeContext   = ctxt
                  , datatypeName      = parentName
-#if MIN_VERSION_th_abstraction(0,3,0)
                  , datatypeInstTypes = instTypes
-#else
-                 , datatypeVars      = instTypes
-#endif
                  , datatypeVariant   = variant
                  , datatypeCons      = cons
                  } -> do
