@@ -112,6 +112,15 @@ data Empty2 a
 type role Empty2 nominal
 #endif
 
+data TyCon29 a
+    = TyCon29a (forall b. b -> (forall c. a -> c) -> a)
+    | TyCon29b (Int -> forall c. c -> a)
+
+type family F :: * -> *
+type instance F = Maybe
+
+data TyCon30 a = TyCon30 (F a)
+
 -- Data families
 
 data family   StrangeFam x  y z
@@ -174,6 +183,14 @@ data family   IntHashFunFam x y
 data instance IntHashFunFam a b
     = IntHashFunFam ((((a -> Int#) -> b) -> Int#) -> a)
 
+data family   TyFamily29 x
+data instance TyFamily29 a
+    = TyFamily29a (forall b. b -> (forall c. a -> c) -> a)
+    | TyFamily29b (Int -> forall c. c -> a)
+
+data family   TyFamily30 x
+data instance TyFamily30 a = TyFamily30 (F a)
+
 -------------------------------------------------------------------------------
 
 -- Plain data types
@@ -195,11 +212,15 @@ $(deriveTraversable ''OneTwoCompose)
 
 instance Functor (f Int Int) => Functor (ComplexConstraint f g a) where
     fmap      = $(makeFmap      ''ComplexConstraint)
+    (<$)      = $(makeReplace   ''ComplexConstraint)
 instance Foldable (f Int Int) => Foldable (ComplexConstraint f g a) where
     foldr     = $(makeFoldr     ''ComplexConstraint)
     foldMap   = $(makeFoldMap   ''ComplexConstraint)
     fold      = $(makeFold      ''ComplexConstraint)
     foldl     = $(makeFoldl     ''ComplexConstraint)
+#if MIN_VERSION_base(4,8,0)
+    null      = $(makeNull      ''ComplexConstraint)
+#endif
 instance Traversable (f Int Int) => Traversable (ComplexConstraint f g a) where
     traverse  = $(makeTraverse  ''ComplexConstraint)
     sequenceA = $(makeSequenceA ''ComplexConstraint)
@@ -227,6 +248,12 @@ $(deriveFunctorOptions     defaultFFTOptions{ fftEmptyCaseBehavior = True } ''Em
 $(deriveFoldableOptions    defaultFFTOptions{ fftEmptyCaseBehavior = True } ''Empty2)
 $(deriveTraversableOptions defaultFFTOptions{ fftEmptyCaseBehavior = True } ''Empty2)
 
+$(deriveFunctor     ''TyCon29)
+
+$(deriveFunctor     ''TyCon30)
+$(deriveFoldable    ''TyCon30)
+$(deriveTraversable ''TyCon30)
+
 #if MIN_VERSION_template_haskell(2,7,0)
 -- Data families
 
@@ -247,11 +274,15 @@ $(deriveTraversable 'OneTwoComposeFam)
 
 instance Functor (f Int Int) => Functor (ComplexConstraintFam f g a) where
     fmap      = $(makeFmap      'ComplexConstraintFam)
+    (<$)      = $(makeReplace   'ComplexConstraintFam)
 instance Foldable (f Int Int) => Foldable (ComplexConstraintFam f g a) where
     foldr     = $(makeFoldr     'ComplexConstraintFam)
     foldMap   = $(makeFoldMap   'ComplexConstraintFam)
     fold      = $(makeFold      'ComplexConstraintFam)
     foldl     = $(makeFoldl     'ComplexConstraintFam)
+# if MIN_VERSION_base(4,8,0)
+    null      = $(makeNull      'ComplexConstraintFam)
+# endif
 instance Traversable (f Int Int) => Traversable (ComplexConstraintFam f g a) where
     traverse  = $(makeTraverse  'ComplexConstraintFam)
     sequenceA = $(makeSequenceA 'ComplexConstraintFam)
@@ -269,6 +300,12 @@ $(deriveFoldable    'IntHashTupleFam)
 $(deriveTraversable 'IntHashFam)
 
 $(deriveFunctor     'IntHashFunFam)
+
+$(deriveFunctor     'TyFamily29a)
+
+$(deriveFunctor     'TyFamily30)
+$(deriveFoldable    'TyFamily30)
+$(deriveTraversable 'TyFamily30)
 #endif
 
 -------------------------------------------------------------------------------
